import {Component, EventEmitter, Input, Output, ViewChild} from '@angular/core'
import {Mix, MixMaterial, MixMaterialDto, mixMaterialsAsMixMaterialsDto, Recipe} from '../../../shared/model/recipe.model'
import {Subject} from 'rxjs'
import {SubscribingComponent} from '../../../shared/components/subscribing.component'
import {convertMixMaterialQuantity, UNIT_MILLILITER} from '../../../shared/units'
import {ActivatedRoute, Router} from '@angular/router'
import {PtouchPrinter} from '../../ptouchPrint'
import {ConfirmBoxComponent} from '../../../shared/components/confirm-box/confirm-box.component'
import {ErrorService} from '../../../shared/service/error.service'
import {AlertService} from '../../../shared/service/alert.service'
import {MaterialService} from '../../../material/service/material.service'
import {Permission} from '../../../shared/model/user'
import {AccountService} from '../../../accounts/services/account.service'
import {Material, openSimdut} from '../../../shared/model/material.model'

@Component({
  selector: 'cre-mix-table',
  templateUrl: './mix-table.component.html',
  styleUrls: ['./mix-table.component.sass']
})
export class MixTableComponent extends SubscribingComponent {
  private readonly COLUMNS = ['material', 'materialType', 'quantity', 'quantityCalculated', 'quantityUnits', 'simdut']
  private readonly COLUMNS_EDIT = ['position', 'material', 'materialType', 'quantityStatic', 'quantityUnits']

  @ViewChild('printingConfirmBox') printingConfirmBox: ConfirmBoxComponent

  @Input() mix: Mix
  @Input() recipe: Recipe
  @Input() units$: Subject<string>
  @Input() deductErrorBody
  @Input() editionMode: boolean
  @Output() locationChange = new EventEmitter<{ id: number, location: string }>()
  @Output() quantityChange = new EventEmitter<MixMaterialDto>()
  @Output() deduct = new EventEmitter<void>()
  @Output() printingErrorChange = new EventEmitter<number>()

  mixColumns = this.COLUMNS
  units = UNIT_MILLILITER
  mixMaterials: MixMaterialDto[] = []
  hoveredMixMaterial: MixMaterial | null

  // BPac printer
  printer: PtouchPrinter | null

  constructor(
    private materialService: MaterialService,
    private accountService: AccountService,
    private alertService: AlertService,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit() {
    super.ngOnInit()

    if (this.editionMode) {
      this.mixColumns = this.COLUMNS_EDIT
    }

    this.mixMaterials = mixMaterialsAsMixMaterialsDto(this.mix)

    this.subscribe(
      this.units$,
      u => this.convertQuantities(u)
    )
  }

  hasSimdut(material: Material): boolean {
    return material.simdutUrl != null
  }

  openSimdut(mixMaterial: MixMaterial) {
    openSimdut(mixMaterial.material)
  }

  changeLocation(event: any) {
    this.locationChange.emit({id: this.mix.id, location: event.target.value})
  }

  changeQuantity(event: any, mixMaterial: MixMaterialDto, isTotal = false) {
    const newQuantity = parseInt(event.target.value)
    // Skip if there if the input is null
    if (!newQuantity) {
      return
    }

    let ratio = 1
    if (!isTotal) {
      const originalQuantity = this.findMixMaterialDto(mixMaterial.materialId)
      ratio = newQuantity / originalQuantity.quantity
    } else {
      ratio = newQuantity / this.getTotalQuantity()
    }
    this.mixMaterials.forEach((q, i) => {
      if (!q.isPercents) {
        q.quantity *= ratio
      }
      this.emitQuantityChangeEvent(i)
    })
  }

  getMixMaterialFromDto(mixMaterialDto: MixMaterialDto): MixMaterial {
    return this.mix.mixMaterials.find(m => m.material.id === mixMaterialDto.materialId)
  }

  getMixMaterialQuantityRounded(mixMaterial: MixMaterialDto): number {
    return this.round(this.findMixMaterialDto(mixMaterial.materialId).quantity)
  }

  getTotalQuantity(index: number = -1): number {
    if (index === -1) {
      index = this.mixMaterials.length - 1
    }
    let totalQuantity = 0
    for (let i = 0; i <= index; i++) {
      totalQuantity += this.calculateQuantity(i)
    }
    return totalQuantity
  }

  getCalculatedQuantityHtml(mixMaterial: MixMaterial, index: number): string {
    const totalQuantity = this.round(this.getTotalQuantity(index))
    const addedQuantity = this.round(this.calculateQuantity(index))
    return `<span class="mix-calculated-quantity">+${addedQuantity}</span> <span class="mix-calculated-quantity">(${totalQuantity})</span>`
  }

  isInLowQuantity(materialId: number): boolean {
    return this.deductErrorBody.mix === this.mix.id &&
      this.deductErrorBody.lowQuantities.filter(l => l.materialId == materialId).length > 0
  }

  round(quantity: number): number {
    return Math.round(quantity * 1000) / 1000
  }

  async print() {
    const base = this.mix.mixMaterials
      .map(ma => ma.material)
      .filter(m => m.materialType.name === 'Base')[0]
    if (!base) {
      this.printingErrorChange.emit(98)
      return
    }
    this.printer = new PtouchPrinter({
      template: 'Couleur',
      lines: [
        {name: 'color', value: this.recipe.name},
        {name: 'banner', value: this.recipe.company.name},
        {name: 'base', value: base.name},
        {name: 'description', value: this.recipe.description}
      ]
    })
    const exitCode = await this.printer.print()
    switch (exitCode) {
      case PrintingExitCode.PrintingInProgress:
        this.alertService.pushSuccess('Impression en cours. Cette opération peut prendre quelques secondes')
        break
      case PrintingExitCode.ExtensionNotInstalled:
        this.alertService.pushError('L\'extension b-Pac n\'est pas installée')
        break
      case PrintingExitCode.NoBaseFound:
        this.alertService.pushError('Il n\'y a pas de base dans ce mélange')
        break
      case PrintingExitCode.GenericPrintingError:
        this.alertService.pushError('Une erreur est survenue durant l\'impression')
        break
      default:
        this.errorService.pushUnknownError()
        break
    }
  }

  get canPrintMixes(): boolean {
    return this.accountService.hasPermission(Permission.PRINT_MIXES)
  }

  get canDeductFromInventory(): boolean {
    return this.accountService.hasPermission(Permission.DEDUCT_FROM_INVENTORY)
  }

  get canEditRecipesPublicData(): boolean {
    return this.accountService.hasPermission(Permission.EDIT_RECIPES_PUBLIC_DATA)
  }

  private emitQuantityChangeEvent(index: number) {
    const quantity = this.mixMaterials[index]
    this.quantityChange.emit({
      materialId: quantity.materialId,
      quantity: this.calculateQuantity(index),
      isPercents: quantity.isPercents,
      position: quantity.position
    })
  }

  private convertQuantities(newUnit: string) {
    this.mixMaterials.forEach(q => q.quantity = convertMixMaterialQuantity(q, this.units, newUnit))
    this.units = newUnit
  }

  private findMixMaterialDto(materialId: number): MixMaterialDto {
    return this.mixMaterials.find(q => q.materialId == materialId)
  }

  private calculateQuantity(index: number): number {
    const computedQuantity = this.mixMaterials[index]
    if (!computedQuantity.isPercents) {
      return computedQuantity.quantity
    }
    return this.mixMaterials[0].quantity * (computedQuantity.quantity / 100)
  }
}

enum PrintingExitCode {
  PrintingInProgress = 1,

  ExtensionNotInstalled = -1,
  NoBaseFound = 98,
  GenericPrintingError = 99
}
