import {Component} from '@angular/core'
import {RecipeService} from '../../services/recipe.service'
import {ActivatedRoute, Router} from '@angular/router'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {MixMaterialDto, Recipe, recipeMixCount, recipeNoteForGroupId, recipeStepCount} from '../../../shared/model/recipe.model'
import {Observable, Subject} from 'rxjs'
import {ErrorHandler, ErrorService} from '../../../shared/service/error.service'
import {AlertService} from '../../../shared/service/alert.service'
import {GlobalAlertHandlerComponent} from '../../../shared/components/global-alert-handler/global-alert-handler.component'
import {InventoryService} from '../../../material/service/inventory.service'
import {ConfirmBoxComponent} from '../../../shared/components/confirm-box/confirm-box.component'
import {GroupService} from '../../../groups/services/group.service'
import {AppState} from '../../../shared/app-state'
import {AccountService} from '../../../accounts/services/account.service'
import {Permission} from '../../../shared/model/user'

@Component({
  selector: 'cre-explore',
  templateUrl: './explore.component.html',
  styleUrls: ['./explore.component.sass']
})
export class ExploreComponent extends ErrorHandlingComponent {
  recipe: Recipe | null
  groups$ = this.groupService.all
  deductErrorBody = {}
  units$ = new Subject<string>()
  selectedGroupId: number | null

  hasModifications = false
  quantitiesChanges = new Map<number, Map<number, number>>()
  mixesLocationChanges = new Map<number, string>()
  groupsNote = new Map<number, string>()

  deductedMixId: number | null

  errorHandlers: ErrorHandler[] = [{
    filter: error => error.type === 'notfound-recipe-id',
    consumer: error => this.urlUtils.navigateTo('/color/list')
  }, {
    filter: error => error.type === 'notenoughinventory-multiple',
    consumer: error => this.deductErrorBody = {mix: this.deductedMixId, lowQuantities: error.lowQuantities},
    messageProducer: () => 'Certains produit ne sont pas en quantité suffisante dans l\'inventaire'
  }]

  constructor(
    private recipeService: RecipeService,
    private inventoryService: InventoryService,
    private groupService: GroupService,
    private alertService: AlertService,
    private accountService: AccountService,
    private appState: AppState,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
  }

  ngOnInit(): void {
    super.ngOnInit()
    GlobalAlertHandlerComponent.extraTopMarginMultiplier = 2.5

    this.selectedGroupId = this.loggedInUserGroupId

    const id = parseInt(this.activatedRoute.snapshot.paramMap.get('id'))
    this.subscribeEntityById(
      this.recipeService,
      id,
      r => {
        this.recipe = r
        this.appState.title = r.name

        if (recipeMixCount(this.recipe) <= 0 || recipeStepCount(this.recipe) <= 0) {
          this.alertService.pushWarning('Cette recette n\'est pas complète')
        }
      }
    )
  }

  ngOnDestroy() {
    super.ngOnDestroy()
    GlobalAlertHandlerComponent.extraTopMarginMultiplier = 0
  }

  changeUnits(unit: string) {
    this.units$.next(unit)
  }

  changeQuantity(event: { mixId: number, mixMaterial: MixMaterialDto }) {
    if (!this.quantitiesChanges.has(event.mixId)) {
      this.quantitiesChanges.set(event.mixId, new Map<number, number>())
    }
    this.quantitiesChanges.get(event.mixId).set(event.mixMaterial.materialId, event.mixMaterial.quantity)
  }

  changeMixLocation(event: { id: number, location: string }) {
    this.hasModifications = true
    this.mixesLocationChanges.set(event.id, event.location)
  }

  saveModifications() {
    this.subscribe(
      this.recipeService.updateExplorerModifications(this.recipe.id, this.mappedUpdatedNotes, this.mixesLocationChanges),
      () => {
        this.hasModifications = false
        this.alertService.pushSuccess('Les modifications ont été enregistrées')
      },
      true
    )
  }

  showDeductMixConfirm(mixId: number, confirmBox: ConfirmBoxComponent) {
    this.deductedMixId = mixId
    confirmBox.show()
  }

  deductMix() {
    const firstMixMaterial = this.recipe.mixes.filter(m => m.id === this.deductedMixId)[0].mixMaterials[0]
    if (this.quantitiesChanges.has(this.deductedMixId) && this.quantitiesChanges.get(this.deductedMixId).has(firstMixMaterial.material.id)) {
      const originalQuantity = firstMixMaterial.quantity
      const currentQuantity = this.quantitiesChanges.get(this.deductedMixId).get(firstMixMaterial.material.id)
      this.subscribeDeductMix(this.inventoryService.deductMixFromQuantities(this.deductedMixId, originalQuantity, currentQuantity))
    } else {
      this.subscribeDeductMix(this.inventoryService.deductMix(this.deductedMixId, 1))
    }
  }

  subscribeDeductMix(observable: Observable<any>) {
    this.subscribe(
      observable,
      () => this.alertService.pushSuccess('Les quantités quantités ont été déduites de l\'inventaire'),
      true
    )
  }

  get loggedInUserGroupId(): number {
    return this.appState.authenticatedUser.group?.id
  }

  get selectedGroupNote(): string {
    if (!this.groupsNote.has(this.selectedGroupId)) {
      this.groupsNote.set(this.selectedGroupId, recipeNoteForGroupId(this.recipe, this.selectedGroupId))
    }
    return this.groupsNote.get(this.selectedGroupId)
  }

  set selectedGroupNote(value: string) {
    this.groupsNote.set(this.selectedGroupId, value)
  }

  get canEditRecipesPublicData(): boolean {
    return this.accountService.hasPermission(Permission.EDIT_RECIPES_PUBLIC_DATA)
  }

  private get mappedUpdatedNotes(): Map<number, string> {
    const updatedNotes = new Map<number, string>()

    this.recipe.groupsInformation.forEach(i => {
      updatedNotes.set(i.group.id, i.note)
    })

    this.groupsNote.forEach((content, groupId) => {
      updatedNotes.set(groupId, content)
    })

    return updatedNotes
  }
}
