import {Component, ViewChild} from '@angular/core'
import {ErrorHandlingComponent} from '../../../shared/components/subscribing.component'
import {MaterialService} from '../../service/material.service'
import {Permission} from '../../../shared/model/user'
import {ActivatedRoute, Router} from '@angular/router'
import {ErrorService} from '../../../shared/service/error.service'
import {Material, openSimdut} from '../../../shared/model/material.model'
import {AccountService} from '../../../accounts/services/account.service'
import {convertQuantity, UNIT_MILLILITER} from '../../../shared/units'
import {MatSort} from '@angular/material/sort'
import {MatTableDataSource} from '@angular/material/table'
import {MaterialTypeService} from '../../../material-type/service/material-type.service'
import {InventoryService} from '../../service/inventory.service'
import {AppState} from '../../../shared/app-state'

@Component({
  selector: 'cre-list',
  templateUrl: './inventory.component.html',
  styleUrls: ['./inventory.component.sass']
})
export class InventoryComponent extends ErrorHandlingComponent {
  @ViewChild(MatSort) sort: MatSort

  materials: Material[] | null
  materialTypes$ = this.materialTypeService.all
  dataSource: MatTableDataSource<Material>

  columns = ['name', 'materialType', 'quantity', 'addQuantity', 'lowQuantityIcon', 'simdutIcon', 'editButton', 'openSimdutButton']
  hoveredMaterial: Material | null
  selectedMaterial: Material | null

  units = UNIT_MILLILITER
  lowQuantityThreshold = 100  // TEMPORARY will be in the application settings
  materialTypeFilter = 1
  materialNameFilter = ''

  constructor(
    private materialService: MaterialService,
    private materialTypeService: MaterialTypeService,
    private inventoryService: InventoryService,
    private accountService: AccountService,
    private appState: AppState,
    errorService: ErrorService,
    router: Router,
    activatedRoute: ActivatedRoute
  ) {
    super(errorService, activatedRoute, router)
    this.appState.title = 'Inventaire'
  }

  ngOnInit() {
    super.ngOnInit()

    this.subscribe(
      this.materialService.allNotMixType,
      materials => {
        this.materials = materials
        this.dataSource = this.setupDataSource()
      },
      true,
      1
    )
  }

  setupDataSource(): MatTableDataSource<Material> {
    this.dataSource = new MatTableDataSource<Material>(this.materials)
    this.dataSource.sortingDataAccessor = (material, header) => {
      switch (header) {
        case 'materialType':
          return material[header].name
        case 'lowQuantityIcon':
          return this.isLowQuantity(material)
        default:
          return material[header]
      }
    }
    this.dataSource.filterPredicate = (material, filter) => {
      return (!this.materialTypeFilter || this.materialTypeFilter === 1 || material.materialType.id === this.materialTypeFilter) &&
        (!this.materialNameFilter || material.name.toLowerCase().includes(this.materialNameFilter.toLowerCase()))
    }

    this.dataSource.sort = this.sort
    return this.dataSource
  }

  filterDataSource() {
    this.dataSource.filter = 'filter'
  }

  isLowQuantity(material: Material) {
    return this.getQuantity(material) < this.lowQuantityThreshold
  }

  getQuantity(material: Material): number {
    return Math.round(convertQuantity(material.inventoryQuantity, UNIT_MILLILITER, this.units) * 100) / 100
  }

  materialHasSimdut(material: Material): boolean {
    return material.simdutUrl != null
  }

  openSimdut(material: Material) {
    openSimdut(material)
  }

  addQuantity(material: Material, input: HTMLInputElement) {
    const quantity = input.valueAsNumber
    this.subscribe(
      this.inventoryService.add(material.id, quantity),
      materialQuantities => {
        // Reset the input value
        input.value = null
        this.selectedMaterial = null

        material.inventoryQuantity = parseInt(materialQuantities.filter(mq => mq.material === material.id)[0].quantity)
      },
      true
    )
  }

  get canEditMaterial(): boolean {
    return this.accountService.hasPermission(Permission.EDIT_MATERIALS)
  }

  get canAddToInventory(): boolean {
    return this.accountService.hasPermission(Permission.ADD_TO_INVENTORY)
  }
}
