import {Component, Input} from '@angular/core'
import {Observable} from 'rxjs'
import {AccountService} from '../../../accounts/services/account.service'
import {Permission} from '../../model/user'
import {animate, state, style, transition, trigger} from '@angular/animations'

@Component({
  selector: 'cre-entity-list',
  templateUrl: './entity-list.component.html',
  styleUrls: ['./entity-list.component.sass'],
  animations: [
    trigger('detailExpand', [
      state('collapsed', style({height: '0px', minHeight: '0'})),
      state('expanded', style({height: '*'})),
      transition('expanded <=> collapsed', animate('225ms cubic-bezier(0.4, 0.0, 0.2, 1)'))
    ])
  ]
})
export class EntityListComponent<T> {
  @Input() entities$: Observable<T>
  @Input() columns: TableColumn[]
  @Input() icons: TableIcon[]
  @Input() buttons?: TableButton[]
  @Input() addLink: string
  @Input() addPermission: Permission
  @Input() expandable = false
  @Input() rowDetailsTemplate

  hoveredEntity: T | null
  expandedEntity: T | null

  constructor(
    private accountService: AccountService
  ) {
  }

  hasPermissionToUseButton(button: TableButton): boolean {
    return !button.permission || this.hasPermission(button.permission)
  }

  hasPermission(permission: Permission): boolean {
    return this.accountService.hasPermission(permission)
  }

  getRouterLink(button: TableButton, entity: T): string {
    // @ts-ignore
    if (button.link && !button.link.externalLink) {
      // @ts-ignore
      return button.link
    } else if (button.linkFn) {
      const fnResult = button.linkFn(entity)
      // @ts-ignore
      if (!fnResult.externalLink) {
        // @ts-ignore
        return fnResult
      }
    }
    return undefined
  }

  clickButton(button: TableButton, entity: T) {
    if (button.link || button.linkFn) {
      let externalLink = null
      // @ts-ignore
      if (button.link && button.link.externalLink) {
        // @ts-ignore
        externalLink = button.link.externalLink
      } else {
        const linkFnResult = button.linkFn(entity)
        // @ts-ignore
        if (linkFnResult && linkFnResult.externalLink) {
          // @ts-ignore
          externalLink = linkFnResult.externalLink
        }
      }

      if (externalLink) {
        window.open(externalLink, '_blank')
      }
    } else if (button.clickFn) {
      button.clickFn(entity)
    }
  }

  get tableCols(): string[] {
    const cols = this.columns.map(c => c.def)
    if (this.icons) {
      this.icons.forEach((_, i) => cols.push(`icon${i}`))
    }
    if (this.buttons) {
      this.buttons.forEach((_, i) => cols.push(`button${i}`))
    }
    return cols
  }
}

export class TableColumn {
  constructor(
    public def: string,
    public title: string,
    public valueFn: (T) => string
  ) {
  }
}

export class TableIcon {
  constructor(
    public icon: string,
    public color: string,
    public title: string,
    public disabledFn: (T) => boolean
  ) {
  }
}

export class TableButton {
  constructor(
    public text: string,
    public link: string | { externalLink: string } | null,
    public linkFn: (T) => string | { externalLink: string } | null,
    public clickFn: (T) => void,
    public permission: Permission | null,
    public disabledFn: (T) => boolean | null
  ) {
  }
}
