import {
  Component,
  ContentChild,
  ElementRef,
  EventEmitter,
  Input,
  OnDestroy,
  OnInit,
  Output,
  TemplateRef,
  ViewChild,
  ViewEncapsulation
} from '@angular/core'
import {AbstractControl, FormControl, ValidationErrors, ValidatorFn} from '@angular/forms'
import {COMMA, ENTER} from '@angular/cdk/keycodes'
import {Observable, Subject} from 'rxjs'
import {map, takeUntil} from 'rxjs/operators'
import {MatChipInputEvent} from '@angular/material/chips'
import {MatAutocomplete, MatAutocompleteSelectedEvent} from '@angular/material/autocomplete'
import {MatFormFieldAppearance} from '@angular/material/form-field'

@Component({
  selector: 'cre-input',
  templateUrl: 'input.html',
  encapsulation: ViewEncapsulation.None,
  styleUrls: ['input.sass']
})
export class CreInputComponent {
  @Input() control: FormControl | null
  @Input() type = 'text'
  @Input() label: string
  @Input() icon: string
  @Input() iconTitle: string | null
  @Input() required = true
  @Input() autocomplete = true
  @Input() placeholder: string | null
  @Input() step = 1
  @Input() value
  @Input() iconColor: string = 'black'
  @Input() disabled = false
  @Input() hint: string | null

  @Output() valueChange = new EventEmitter<any>()

  @ContentChild(TemplateRef) errors: TemplateRef<any>
}

@Component({
  selector: 'cre-autocomplete-input',
  templateUrl: 'autocomplete-input.html',
  encapsulation: ViewEncapsulation.None
})
export class CreAutocompleteInputComponent {
  @Input() control: FormControl | null
  @Input() label: string
  @Input() icon: string
  @Input() required = true
  @Input() options: Observable<string[]>
  @Input() value

  @Output() valueChange = new EventEmitter<any>()

  @ContentChild(TemplateRef) errors: TemplateRef<any>
}

@Component({
  selector: 'cre-chip-input',
  templateUrl: 'chip-input.html',
  encapsulation: ViewEncapsulation.None
})
export class CreChipInputComponent implements OnInit {
  @Input() control: FormControl
  @Input() label: string
  @Input() icon: string
  @Input() required = true

  inputControl = new FormControl()

  readonly separatorKeysCodes = [ENTER, COMMA]

  selectedValues = []

  ngOnInit(): void {
    if (this.control.value) {
      this.selectedValues = this.control.value
    }
  }

  add(event: MatChipInputEvent): void {
    const input = event.input
    const value = event.value

    if ((value || '').trim()) {
      this.selectedValues.push(value.trim())
    }

    if (input) {
      input.value = ''
    }

    this.inputControl.setValue(null)
    this.control.setValue(this.selectedValues)
  }

  remove(value): void {
    const index = this.selectedValues.indexOf(value)

    if (index >= 0) {
      this.selectedValues.splice(index, 1)
      this.control.setValue(this.selectedValues)
    }
  }
}

@Component({
  selector: 'cre-combo-box',
  templateUrl: 'combo-box.html',
  encapsulation: ViewEncapsulation.None
})
export class CreComboBoxComponent {
  @Input() control: FormControl
  @Input() label: string
  @Input() icon: string
  @Input() required = true
  @Input() options: Observable<ComboBoxEntry[]>

  @ContentChild(TemplateRef) errors: TemplateRef<any>
}

@Component({
  selector: 'cre-chip-combo-box',
  templateUrl: 'chip-combo-box.html',
  encapsulation: ViewEncapsulation.None
})
export class CreChipComboBoxComponent extends CreChipInputComponent implements OnDestroy {
  @Input() options: Observable<ComboBoxEntry[]>

  @ContentChild(TemplateRef) errors: TemplateRef<any>
  @ViewChild('chipInput') chipInput: ElementRef<HTMLInputElement>
  @ViewChild('auto') matAutocomplete: MatAutocomplete

  filteredOptions: Observable<ComboBoxEntry[]>

  private _options: ComboBoxEntry[]
  private _destroy$ = new Subject()

  ngOnInit() {
    super.ngOnInit()

    this.options.pipe(takeUntil(this._destroy$))
      .subscribe({next: options => this._options = options})

    this.filteredOptions = this.inputControl.valueChanges.pipe(
      map((query: string | null) => query ? this._filter(query) : this._options.slice())
    )
  }

  ngOnDestroy(): void {
    this._destroy$.complete()
  }

  selected(event: MatAutocompleteSelectedEvent) {
    this.selectedValues.push(this.findValueByKey(event.option.value))
    this.chipInput.nativeElement.value = ''
    this.inputControl.setValue(null)
    this.control.setValue(this.selectedValues)
  }

  get empty(): boolean {
    return this.selectedValues.length <= 0
  }

  private _filter(query: string): ComboBoxEntry[] {
    const filterValue = query.toString().toLowerCase()
    return this._options.filter(option => option.value.toString().toLowerCase().indexOf(filterValue) === 0)
  }

  private findValueByKey(key: any): any {
    return this._options.filter(o => o.key === key)[0].value
  }
}

@Component({
  selector: 'cre-checkbox-input',
  templateUrl: 'checkbox.html'
})
export class CreCheckboxInputComponent {
  @Input() label: string
  @Input() control: FormControl
  @Input() checked: boolean

  @Output() checkedChange = new EventEmitter<boolean>()
}

@Component({
  selector: 'cre-file-input',
  templateUrl: 'file-input.html'
})
export class CreFileInputComponent implements OnInit {
  @Input() label: string
  @Input() icon: string
  @Input() accept = ''
  @Input() control: FormControl | null

  @Output() selection = new EventEmitter<File>()
  @Output() invalidFormat = new EventEmitter<void>()

  private acceptedMediaTypes: string[]

  ngOnInit(): void {
    this.acceptedMediaTypes = this.accept.split(',')
  }

  selectedFile: File | null
  selectedFileName: string

  onFileSelected(event: any) {
    this.selectedFile = event.target.files[0]
    if (this.acceptedMediaTypes.indexOf(this.selectedFile.type) >= 0) {
      this.selectedFileName = this.selectedFile.name
      this.control?.setValue(this.selectedFile)
      this.control?.markAsDirty()
      this.selection.emit(this.selectedFile)
    } else {
      this.invalidFormat.emit()
    }
  }
}

export class ComboBoxEntry {
  constructor(
    public key: any,
    public value: any,
    public display?: any
  ) {
  }
}

export function chipListRequired(): ValidatorFn {
  return (control: AbstractControl): ValidationErrors | null => {
    return !control.value || control.value.length <= 0 ? {required: true} : null
  }
}
